@@ -21,6 +21,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <stdbool.h>
 
 static AV_LOCK_DECL(initlock);
 static int inited;
@@ -429,6 +430,18 @@ static void get_new_name(struct parse_state *ps, struct ext_info *ext)
         ps->prevseg = av_stradd(ps->prevseg, ext->to, NULL);
 }
 
+static bool avfs_is_filter_like( struct avfs *avfs )
+{
+    if (avfs->flags & AVF_ONLYROOT) {
+        return false;
+    }
+
+    if (av_avfs_implements_readdir(avfs)) {
+        return false;
+    }
+
+    return true;
+}
 
 static int lookup_auto_avfs(struct parse_state *ps, const char *opts,
                             const char *param)
@@ -445,9 +458,14 @@ static int lookup_auto_avfs(struct parse_state *ps, const char *opts,
         av_ref_obj(avfs);
 
         get_new_name(ps, ext);
-        if(find_auto_avfs(ps->prevseg, &ext) == NULL)
+
+        // repeat searching for next avfs if the current avfs is a
+        // filter. Otherwise it does not make sense to use another
+        // avfs. Dir-based vfs need to be entered first.
+
+        if(!avfs_is_filter_like(avfs) || find_auto_avfs(ps->prevseg, &ext) == NULL) { 
             set_prevseg(ps, "");
-        else {
+        } else {
             param = "";
             ps->nextseg = 0;
         }
@@ -578,7 +596,7 @@ static int lookup_segment(struct parse_state *ps, int noavfs)
 static int segment_islocal(struct parse_state *ps, unsigned int seglen )
 {
     int islocal = 0;
-    char c;
+    //char c;
     struct parse_state tempps;
     int f;
 
@@ -588,7 +606,7 @@ static int segment_islocal(struct parse_state *ps, unsigned int seglen )
     av_copy_parsestate( ps, &tempps );
     
     tempps.nextseg = seglen;
-    c = tempps.path[seglen];
+    //c = tempps.path[seglen];
     tempps.path[seglen] = '\0';
     
     /* we force lookup_segment() to not enter any avfs hierarchie */
